"""
Use this to create a parent class all hippocrates can use
"""

import typing as t
from datetime import datetime
from os import mkdir, path

from hippocrates.questionnaires.models import Answer, QuestionAnswerSet, Result
from pick import pick
from tabulate import tabulate

HOME_DIRECTORY = path.expanduser('~')
STORAGE_DIRECTORY_NAME = '.hippocrates'
STORAGE_FILE_PATH = path.join(HOME_DIRECTORY, STORAGE_DIRECTORY_NAME)
STORAGE_FILE = path.join(STORAGE_FILE_PATH, 'results.csv')


def ask_question_using_pick(options: t.List[Answer],
                            question_from_set: QuestionAnswerSet):
    option, index = pick(
        options, question_from_set.question.text,
        indicator='=>', default_index=0,
    )
    question_from_set.answer = question_from_set.answer_options[index]


class Assessment:
    question_set: t.List = []
    results: t.List = []
    current_question: int = 0

    def questions(self):
        return self.question_set

    def take_assessment(self, interactive=True):
        for question in self.question_set:
            options: t.List = []
            for answer in question.answer_options:
                options.append(answer.text)

            if interactive:
                ask_question_using_pick(options, question)

    def ask_question(self) -> QuestionAnswerSet:
        question_to_ask = self.question_set[self.current_question]
        self.current_question = self.current_question + 1

        return question_to_ask

    def total_score(self):
        total = 0
        for question in self.question_set:
            if not question.answer:
                # raise exception?
                continue
            total = total + question.answer.value
        return total

    def result(self) -> Result:
        total = 0
        for question in self.question_set:
            if not question.answer:
                # raise exception?
                continue
            total = total + question.answer.value

        for result in self.results:
            if result.min <= total <= result.max:
                return result
        raise Exception

    def complete(self):
        """
        check if all questions have an answer
        :return:
        """
        count = 0
        for question in self.question_set:
            if question.answer:
                count = count + 1

        return True if count == self.total_questions else False

    def get_results(self) -> list:
        if not self.complete():
            raise AssessmentIncomplete()

        answer_detail = []
        for question_answer in self.question_set:  # type Question
            answer_detail.append([question_answer.question.text,
                                 question_answer.answer.text])
        return answer_detail

    def display_answers(self):
        table_data = [
            ['Question', 'Answer'],
        ]
        table_data = table_data + self.get_results()
        return tabulate(table_data, tablefmt='fancy_grid')

    def display_result(self):
        result = self.result()
        table_data = [
            ['Result', 'Severity'],
            [result.comment, result.severity]
        ]
        return tabulate(table_data, tablefmt='fancy_grid')

    def check_directory_exists(self):
        if path.isdir(STORAGE_FILE_PATH):
            return True
        mkdir(STORAGE_FILE_PATH)
        return False

    def _write_to_file(self, write_header, score):
        # dir exists but the file might have been deleted
        if not path.isfile(STORAGE_FILE):
            write_header = True

        with open(STORAGE_FILE, 'a') as storage_file:
            if write_header:
                storage_file.write(f'Date Taken,Assessment Name,Score\n')
            storage_file.write(
                f'{datetime.now().strftime("%Y-%m-%d %H:%M:%S")},'
                f'{self.name},{score}\n')

    def save_results(self):
        score = self.total_score()
        write_header = False
        try:
            # create dir and print the headers for the csv file
            if not self.check_directory_exists():
                write_header = True

            self._write_to_file(write_header=write_header, score=score)

        except OSError:
            print(f'Creation of the directory {STORAGE_FILE_PATH} failed')


class AssessmentIncomplete(Exception):
    pass



import random
from unittest.mock import MagicMock

import pytest
from hippocrates.questionnaires.base import Assessment
from hippocrates.questionnaires.beck_depression_index.assessment import \
    BeckDepressionIndexAssessment
from hippocrates.questionnaires.gad2.assessment import GAD2Assessment
from hippocrates.questionnaires.gad7.assessment import GAD7Assessment
from hippocrates.questionnaires.models import (Answer, Question,
                                               QuestionAnswerSet, Result)
from hippocrates.questionnaires.phq2 import PHQ2Assessment
from hippocrates.questionnaires.phq9 import PHQ9Assessment
from hippocrates.questionnaires.rosenberg_self_esteem.assessment import \
    RosenbergSelfEsteemAssessment

ASSESSMENT_LIST = [
    PHQ2Assessment(),
    PHQ9Assessment(),
    GAD2Assessment(),
    GAD7Assessment(),
    RosenbergSelfEsteemAssessment(),
    BeckDepressionIndexAssessment(),
]


def test_assessment_complete():
    assessment = Assessment()

    question = MagicMock()
    question.answer = True
    assessment.total_questions = 1
    assessment.question_set = [question]
    assert assessment.complete()


def test_assessment_result():
    assessment = Assessment()

    question_one = Question('Do you like chips?')
    question_two = Question('Do you like dip?')

    answer_one = Answer('yes', 10)
    answer_two = Answer('no', 3)

    question_set_one = QuestionAnswerSet(question=question_one,
                                         answer_options=[answer_one],
                                         answer=answer_one)
    question_set_two = QuestionAnswerSet(question=question_two,
                                         answer_options=[answer_two],
                                         answer=answer_two)

    assessment.total_questions = 1
    assessment.question_set = [question_set_one, question_set_two]

    result_one = Result(min_score=0, max_score=5, severity='good',
                        comment='OK')
    result_two = Result(min_score=5, max_score=20, severity='not good',
                        comment='NOT OK')
    assessment.results = [result_one, result_two]
    assert assessment.result() == result_two


def test_result_failure(questionnaire_set):
    assessment = Assessment()
    assessment.total_questions = 5
    assessment.question_set = questionnaire_set[0]

    with pytest.raises(Exception):
        assessment.result()


@pytest.mark.parametrize(
    'assessment',
    ASSESSMENT_LIST
)
def test_take_assessment_code(assessment):
    _take_assessment(assessment)
    assert assessment.result()


def _take_assessment(assessment: Assessment):
    while not assessment.complete():
        question: QuestionAnswerSet = assessment.ask_question()
        question.answer_question(random.choice(question.answer_options))


@pytest.mark.parametrize(
    'assessment',
    ASSESSMENT_LIST
)
def test_get_results(assessment):
    _take_assessment(assessment)
    assert len(assessment.get_results()) == len(assessment.question_set)


@pytest.mark.parametrize(
    'assessment',
    ASSESSMENT_LIST
)
def test_display_results(assessment):
    _take_assessment(assessment)
    table = assessment.display_answers()
    assert len(table.split('\n')) > len(assessment.question_set)



from hippocrates.questionnaires.beck_depression_index.assessment import \
    BeckDepressionIndexAssessment


def test_bdi_assessment():
    bdi_assessment = BeckDepressionIndexAssessment()
    assert bdi_assessment.question_set



from hippocrates.questionnaires.models import (Questionnaire, Result,
                                               ResultSet, create_questions)


def test_create_questions(questionnaire_set_json):
    question_set = create_questions(questionnaire_set_json)
    assert len(question_set) == len(questionnaire_set_json['question_set'])


def test_questionnaire_iteration(questionnaire_set_json):
    question_set = create_questions(questionnaire_set_json)
    questionnaire = Questionnaire(question_set)

    for question in questionnaire.questions():
        assert hasattr(question, 'answer_options')
        assert hasattr(question, 'question')


def test_result_set():
    result = Result(
        min_score=0,
        max_score=4,
        severity='sev',
        comment='test comment',
    )
    result_set = ResultSet(results=[result])
    assert result_set



import setuptools

import hippocrates

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='hippocrates',
    version=hippocrates.__version__,
    author='Chris Hannam',
    author_email='ch@chrishannam.co.uk',
    description='Mental health questionnaires.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/chrishannam/hippocrates',
    packages=setuptools.find_packages(exclude=('tests',)),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'click==7.0',
        'pick>=0.6.4',
        'tabulate>=0.8.2',
    ],
    scripts=[
        'bin/hip-take-phq9',
        'bin/hip-take-phq2',
        'bin/hip-take-gad2',
        'bin/hip-take-gad7',
        'bin/hip-take-beck-depression-index',
        'bin/hip-take-rosenberg-self-esteem',
    ],
    include_package_data=True,
)
